from pathlib import Path

from naas_abi_cli.cli.deploy.local import _build_nexus_api_url, setup_local_deploy


def test_build_nexus_api_url_uses_abi_port_for_localhost() -> None:
    assert _build_nexus_api_url("localhost") == "http://localhost:9879"


def test_build_nexus_api_url_uses_abi_port_for_ip_address() -> None:
    assert _build_nexus_api_url("127.0.0.1") == "http://127.0.0.1:9879"


def test_build_nexus_api_url_keeps_explicit_port() -> None:
    assert _build_nexus_api_url("localhost:8080") == "http://localhost:8080"


def test_build_nexus_api_url_keeps_domain_without_port() -> None:
    assert _build_nexus_api_url("api.localhost") == "https://api.localhost"


def test_build_nexus_api_url_allows_scheme_override() -> None:
    assert _build_nexus_api_url("api.localhost", public_api_scheme="http") == (
        "http://api.localhost"
    )


def test_setup_local_deploy_does_not_include_headscale_by_default(
    tmp_path: Path,
) -> None:
    setup_local_deploy(str(tmp_path), base_domain="localhost")

    compose_path = tmp_path / "docker-compose.yml"
    env_path = tmp_path / ".env"

    compose_content = compose_path.read_text(encoding="utf-8")
    env_content = env_path.read_text(encoding="utf-8")

    assert "  headscale:" not in compose_content
    assert "HEADSCALE_SERVER_PORT=" not in env_content


def test_setup_local_deploy_can_include_headscale(tmp_path: Path) -> None:
    setup_local_deploy(
        str(tmp_path),
        include_headscale=True,
        base_domain="localhost",
    )

    compose_path = tmp_path / "docker-compose.yml"
    env_path = tmp_path / ".env"
    headscale_config_path = tmp_path / ".deploy/docker/headscale/config.yaml"
    headscale_extra_records_path = (
        tmp_path / ".deploy/docker/headscale/extra-records.json"
    )

    compose_content = compose_path.read_text(encoding="utf-8")
    env_content = env_path.read_text(encoding="utf-8")

    assert "  headscale:" in compose_content
    assert "headscale_data:" in compose_content
    assert "HEADSCALE_SERVER_PORT=" in env_content
    assert headscale_config_path.exists()
    assert headscale_extra_records_path.exists()


def test_setup_local_deploy_uses_selected_hosts_for_generated_env(
    tmp_path: Path,
) -> None:
    setup_local_deploy(
        str(tmp_path),
        base_domain="example.com",
    )

    env_content = (tmp_path / ".env").read_text(encoding="utf-8")

    assert "PUBLIC_WEB_HOST=nexus.example.com" in env_content
    assert "PUBLIC_API_HOST=api.example.com" in env_content
    assert "NEXUS_API_URL=https://api.example.com" in env_content


def test_setup_local_deploy_generates_headscale_and_vpn_domains(tmp_path: Path) -> None:
    setup_local_deploy(
        str(tmp_path),
        include_headscale=True,
        base_domain="example.com",
    )

    env_content = (tmp_path / ".env").read_text(encoding="utf-8")

    assert "HEADSCALE_SERVER_URL=headscale.example.com" in env_content
    assert "HEADSCALE_INTERNAL_DOMAIN=vpn.example.com" in env_content


def test_setup_local_deploy_regenerate_creates_backup_and_refreshes_compose(
    tmp_path: Path,
) -> None:
    setup_local_deploy(str(tmp_path), base_domain="localhost")

    compose_path = tmp_path / "docker-compose.yml"
    env_path = tmp_path / ".env"
    compose_path.write_text("custom compose content\n", encoding="utf-8")
    env_path.write_text(
        "PUBLIC_WEB_HOST=custom.local\nCUSTOM_VAR=kept\n",
        encoding="utf-8",
    )

    setup_local_deploy(
        str(tmp_path),
        base_domain="localhost",
        regenerate=True,
    )

    backup_base = tmp_path / ".abi-backups/deploy-local"
    backup_dirs = list(backup_base.iterdir())

    assert len(backup_dirs) == 1
    assert (backup_dirs[0] / "docker-compose.yml").read_text(encoding="utf-8") == (
        "custom compose content\n"
    )
    assert "services:" in compose_path.read_text(encoding="utf-8")

    refreshed_env_content = env_path.read_text(encoding="utf-8")
    assert "PUBLIC_WEB_HOST=custom.local" in refreshed_env_content
    assert "CUSTOM_VAR=kept" in refreshed_env_content


def test_setup_local_deploy_regenerate_without_backup(tmp_path: Path) -> None:
    setup_local_deploy(str(tmp_path), base_domain="localhost")

    setup_local_deploy(
        str(tmp_path),
        base_domain="localhost",
        regenerate=True,
        backup=False,
    )

    assert not (tmp_path / ".abi-backups").exists()



import hashlib
import re
from dataclasses import dataclass
from datetime import date, datetime, timedelta
from typing import Dict, List, Optional

import requests
from naas_abi_core import logger
from naas_abi_core.integration import Integration, IntegrationConfiguration
from naas_abi_core.integration.integration import IntegrationConnectionError
from naas_abi_core.services.cache.CacheFactory import CacheFactory
from naas_abi_core.services.cache.CachePort import DataType
from naas_abi_core.services.cache.CacheService import CacheService


@dataclass
class {{integration_name_pascal}}IntegrationConfiguration(IntegrationConfiguration):
    """Configuration for the {{integration_name_pascal}} integration."""
    pass


class {{integration_name_pascal}}Integration(Integration):
    """{{integration_name_pascal}} integration."""

    __configuration: {{integration_name_pascal}}IntegrationConfiguration

    def __init__(self, configuration: {{integration_name_pascal}}IntegrationConfiguration):
        super().__init__(configuration)
        self.__configuration = configuration




from naas_abi_cli.cli import stack_services
from naas_abi_cli.cli.stack_runtime import ComposeServiceState
from naas_abi_cli.cli.stack_services import evaluate_service_readiness


def test_readiness_uses_docker_health_when_healthy() -> None:
    state = ComposeServiceState(
        service="postgres",
        container_name="project-postgres-1",
        state="running",
        health="healthy",
        exit_code=None,
        status="Up",
    )

    readiness = evaluate_service_readiness("postgres", state)
    assert readiness.ready is True
    assert readiness.source == "docker-health"


def test_readiness_detects_one_shot_completed() -> None:
    state = ComposeServiceState(
        service="minio-init",
        container_name="project-minio-init-1",
        state="exited",
        health=None,
        exit_code=0,
        status="Exited (0)",
    )

    readiness = evaluate_service_readiness("minio-init", state)
    assert readiness.ready is True


def test_readiness_is_false_when_not_running_without_health() -> None:
    state = ComposeServiceState(
        service="qdrant",
        container_name="project-qdrant-1",
        state="created",
        health=None,
        exit_code=None,
        status="Created",
    )

    readiness = evaluate_service_readiness("qdrant", state)
    assert readiness.ready is False
    assert readiness.source == "compose"


def test_readiness_for_abi_requires_port_9879(monkeypatch) -> None:
    state = ComposeServiceState(
        service="abi",
        container_name="project-abi-1",
        state="running",
        health="healthy",
        exit_code=None,
        status="Up",
    )

    monkeypatch.setattr(
        stack_services, "_check_http", lambda url, timeout=1.5: (False, "down")
    )

    readiness = evaluate_service_readiness("abi", state)
    assert readiness.ready is False
    assert readiness.source == "http"


def test_readiness_for_abi_is_true_when_port_9879_is_alive(monkeypatch) -> None:
    state = ComposeServiceState(
        service="abi",
        container_name="project-abi-1",
        state="running",
        health="healthy",
        exit_code=None,
        status="Up",
    )

    monkeypatch.setattr(
        stack_services, "_check_http", lambda url, timeout=1.5: (True, "HTTP 200")
    )

    readiness = evaluate_service_readiness("abi", state)
    assert readiness.ready is True
    assert readiness.source == "http"



import click
from naas_abi_core.engine.Engine import Engine
from rich.console import Console
from rich.table import Table


@click.group("agent")
def agent():
    pass


@agent.command("list")
def list():
    engine = Engine()
    engine.load()

    console = Console()
    table = Table(
        title="Available Agents", show_header=True, header_style="bold magenta"
    )
    table.add_column("Module", style="cyan", no_wrap=True)
    table.add_column("Agent", style="green")

    modules = engine.modules

    # Sort modules alphabetically
    sorted_modules = sorted(modules.keys())

    for module in sorted_modules:
        # Sort agents alphabetically by name
        sorted_agents = sorted(modules[module].agents, key=lambda agent: agent.__name__)
        for agent in sorted_agents:
            table.add_row(module, agent.__name__)

    console.print(table)



from naas_abi_cli.cli.stack_runtime import ComposeServiceState, _parse_ps_json


def test_parse_ps_json_list_payload() -> None:
    payload = '[{"Service":"postgres","State":"running"}]'
    rows = _parse_ps_json(payload)
    assert len(rows) == 1
    assert rows[0]["Service"] == "postgres"


def test_parse_ps_json_single_payload() -> None:
    payload = '{"Service":"redis","State":"running"}'
    rows = _parse_ps_json(payload)
    assert len(rows) == 1
    assert rows[0]["Service"] == "redis"


def test_parse_ps_json_line_delimited_payload() -> None:
    payload = "\n".join(
        [
            '{"Service":"redis","State":"running"}',
            '{"Service":"postgres","State":"running"}',
        ]
    )
    rows = _parse_ps_json(payload)
    assert len(rows) == 2


def test_compose_service_state_dataclass() -> None:
    state = ComposeServiceState(
        service="abi",
        container_name="proj-abi-1",
        state="running",
        health="healthy",
        exit_code=None,
        status="Up 2m",
    )
    assert state.service == "abi"
    assert state.health == "healthy"



import os
import shutil
from datetime import datetime
from ipaddress import ip_address
from uuid import uuid4

import naas_abi_cli
from rich.prompt import Prompt

from ..utils.Copier import Copier

LOCAL_ENV_MARKER = "# Added by abi deploy local command execution"
HEADSCALE_SERVICE_MARKER = "  headscale:"
BACKUP_DIRECTORY = os.path.join(".abi-backups", "deploy-local")

DEFAULT_ENV_VALUES: dict[str, str] = {
    "BASE_DOMAIN": "localhost",
    "POSTGRES_USER": "abi",
    "POSTGRES_DB": "abi",
    "POSTGRES_HOST": "postgres",
    "POSTGRES_PORT": "5432",
    "QDRANT_HOST": "qdrant",
    "QDRANT_PORT": "6333",
    "REDIS_PORT": "6379",
    "MINIO_HOST": "minio",
    "MINIO_PORT": "9000",
    "MINIO_ROOT_USER": "abi",
    "ABI_HOST": "abi",
    "PUBLIC_WEB_HOST": "nexus.localhost",
    "PUBLIC_API_HOST": "api.localhost",
    "RABBITMQ_USER": "abi",
    "NEXUS_WEB_IMAGE": "ghcr.io/jupyter-naas/nexus-web",
    "NEXUS_WEB_TAG": "latest",
    "NEXUS_WEB_PORT": "3042",
    "HEADSCALE_SERVER_URL": "headscale.localhost",
    "HEADSCALE_SERVER_PORT": "8083",
    "HEADSCALE_METRICS_PORT": "9090",
    "HEADSCALE_GRPC_PORT": "50443",
    "HEADSCALE_INTERNAL_DOMAIN": "vpn.localhost",
    "HEADSCALE_ACME_EMAIL": "",
}

RANDOM_ENV_KEYS: tuple[str, ...] = (
    "POSTGRES_PASSWORD",
    "MINIO_ROOT_PASSWORD",
    "RABBITMQ_PASSWORD",
    "FUSEKI_ADMIN_PASSWORD",
)

HEADSCALE_DOCKER_COMPOSE_SNIPPET = """

  headscale:
    image: docker.io/headscale/headscale:stable
    pull_policy: always
    restart: unless-stopped
    command: ["headscale", "serve"]
    ports:
      - ${HEADSCALE_SERVER_PORT}:8080
      - ${HEADSCALE_METRICS_PORT}:9090
      - ${HEADSCALE_GRPC_PORT}:50443
    volumes:
      - ./.deploy/docker/headscale/config.yaml:/etc/headscale/config.yaml:ro
      - ./.deploy/docker/headscale/extra-records.json:/var/lib/headscale/extra-records.json:ro
      - headscale_data:/var/lib/headscale
      - headscale_run:/var/run/headscale
    healthcheck:
      test: ["CMD", "headscale", "health"]
      interval: 15s
      timeout: 5s
      retries: 10
      start_period: 30s
    networks:
      - abi-network
"""

HEADSCALE_CONFIG_CONTENT_TEMPLATE = """server_url: https://{headscale_server_url}:{headscale_server_port}
listen_addr: 0.0.0.0:8080
metrics_listen_addr: 0.0.0.0:9090
grpc_listen_addr: 0.0.0.0:50443

prefixes:
  v4: 100.64.0.0/10
  v6: fd7a:115c:a1e0::/48

noise:
  private_key_path: /var/lib/headscale/noise_private.key

database:
  type: sqlite
  sqlite:
    path: /var/lib/headscale/db.sqlite

dns:
  base_domain: {headscale_internal_domain}
  extra_records_path: /var/lib/headscale/extra-records.json

log:
  level: info
"""


def _copy_headscale_templates(deploy_path: str, values: dict[str, object]) -> None:
    copier = Copier(
        templates_path=os.path.join(
            os.path.dirname(naas_abi_cli.__file__),
            "cli/deploy/templates/local/docker/headscale",
        ),
        destination_path=os.path.join(deploy_path, "docker/headscale"),
    )
    copier.copy(values=values)

    headscale_config_path = os.path.join(deploy_path, "docker/headscale/config.yaml")
    if not os.path.exists(headscale_config_path):
        headscale_server_url = str(
            values.get(
                "HEADSCALE_SERVER_URL",
                DEFAULT_ENV_VALUES["HEADSCALE_SERVER_URL"],
            )
        )
        headscale_server_port = str(
            values.get(
                "HEADSCALE_SERVER_PORT",
                DEFAULT_ENV_VALUES["HEADSCALE_SERVER_PORT"],
            )
        )
        headscale_internal_domain = str(
            values.get(
                "HEADSCALE_INTERNAL_DOMAIN",
                DEFAULT_ENV_VALUES["HEADSCALE_INTERNAL_DOMAIN"],
            )
        )
        config_content = HEADSCALE_CONFIG_CONTENT_TEMPLATE.format(
            headscale_server_url=headscale_server_url,
            headscale_server_port=headscale_server_port,
            headscale_internal_domain=headscale_internal_domain,
        )
        with open(headscale_config_path, "w", encoding="utf-8") as config_file:
            config_file.write(config_content)


def _ensure_headscale_service(docker_compose_target_path: str) -> None:
    if not os.path.exists(docker_compose_target_path):
        return

    with open(docker_compose_target_path, "r", encoding="utf-8") as compose_file:
        compose_content = compose_file.read()

    if HEADSCALE_SERVICE_MARKER in compose_content:
        return

    volumes_anchor = "\nvolumes:\n"
    if volumes_anchor not in compose_content:
        raise ValueError(
            "Unable to add headscale service: docker compose volumes section missing"
        )

    compose_with_service = compose_content.replace(
        volumes_anchor,
        HEADSCALE_DOCKER_COMPOSE_SNIPPET + volumes_anchor,
        1,
    )

    if "  headscale_data:" not in compose_with_service:
        compose_with_service = compose_with_service.replace(
            "  qdrant_storage: # Qdrant vector store data\n",
            "  qdrant_storage: # Qdrant vector store data\n"
            "  headscale_data: # Headscale state and sqlite database\n"
            "  headscale_run:  # Headscale unix socket runtime data\n",
            1,
        )

    with open(docker_compose_target_path, "w", encoding="utf-8") as compose_file:
        compose_file.write(compose_with_service)


def _split_host_and_port(host: str) -> tuple[str, str | None]:
    normalized_host = host.strip()

    if normalized_host.startswith("["):
        bracket_end = normalized_host.find("]")
        if bracket_end == -1:
            return normalized_host, None

        host_part = normalized_host[1:bracket_end]
        remainder = normalized_host[bracket_end + 1 :]
        if remainder.startswith(":") and remainder[1:].isdigit():
            return host_part, remainder[1:]
        return host_part, None

    host_parts = normalized_host.rsplit(":", 1)
    if len(host_parts) == 2 and host_parts[1].isdigit() and ":" not in host_parts[0]:
        return host_parts[0], host_parts[1]

    return normalized_host, None


def _is_localhost_or_ip(host: str) -> bool:
    host_without_port, _ = _split_host_and_port(host)

    if host_without_port.lower() == "localhost":
        return True

    try:
        ip_address(host_without_port)
        return True
    except ValueError:
        return False


def _build_nexus_api_url(
    public_api_host: str,
    public_api_scheme: str | None = None,
    abi_port: str = "9879",
) -> str:
    scheme = public_api_scheme
    if scheme is None:
        scheme = "http" if _is_localhost_or_ip(public_api_host) else "https"

    _, explicit_port = _split_host_and_port(public_api_host)

    if explicit_port is not None:
        return f"{scheme}://{public_api_host}"

    if _is_localhost_or_ip(public_api_host):
        return f"{scheme}://{public_api_host}:{abi_port}"

    return f"{scheme}://{public_api_host}"


def _build_hosts_from_domain(domain: str) -> dict[str, str]:
    normalized_domain = domain.strip().lower()
    return {
        "PUBLIC_WEB_HOST": f"nexus.{normalized_domain}",
        "PUBLIC_API_HOST": f"api.{normalized_domain}",
        "HEADSCALE_SERVER_URL": f"headscale.{normalized_domain}",
        "HEADSCALE_INTERNAL_DOMAIN": f"vpn.{normalized_domain}",
    }


def _print_domain_recap(
    base_domain: str,
    public_api_host: str,
    public_web_host: str,
    nexus_api_url: str,
    include_headscale: bool,
    headscale_server_url: str,
    headscale_internal_domain: str,
) -> None:
    print("\nLocal deploy domain recap")
    print(f"- base domain: {base_domain}")
    print(f"- API host: {public_api_host}")
    print(f"- Nexus host: {public_web_host}")
    print(f"- NEXUS_API_URL: {nexus_api_url}")
    if include_headscale:
        print(f"- Headscale host: {headscale_server_url}")
        print(f"- VPN domain: {headscale_internal_domain}")
    print()


def _ensure_env_var(env_path: str, key: str, value: str) -> None:
    existing_content = ""
    if os.path.exists(env_path):
        with open(env_path, "r", encoding="utf-8") as env_file:
            existing_content = env_file.read()

    for line in existing_content.splitlines():
        if line.strip().startswith(f"{key}="):
            return

    with open(env_path, "a", encoding="utf-8") as env_file:
        if existing_content and not existing_content.endswith("\n"):
            env_file.write("\n")
        env_file.write(f"{key}={value}\n")


def _get_env_var(env_path: str, key: str) -> str | None:
    if not os.path.exists(env_path):
        return None

    with open(env_path, "r", encoding="utf-8") as env_file:
        for line in env_file:
            stripped = line.strip()
            if not stripped or stripped.startswith("#"):
                continue
            if not stripped.startswith(f"{key}="):
                continue
            return stripped.split("=", 1)[1]

    return None


def _append_local_env_once(source_env_path: str, destination_env_path: str) -> None:
    with open(source_env_path, "r", encoding="utf-8") as source_file:
        source_content = source_file.read()

    if source_content.strip() == "":
        open(destination_env_path, "a", encoding="utf-8").close()
        return

    current_content = ""
    if os.path.exists(destination_env_path):
        with open(destination_env_path, "r", encoding="utf-8") as destination_file:
            current_content = destination_file.read()

    if LOCAL_ENV_MARKER in current_content:
        return

    with open(destination_env_path, "a", encoding="utf-8") as destination_file:
        if current_content and not current_content.endswith("\n"):
            destination_file.write("\n")
        destination_file.write(source_content)


def _read_env_vars(env_path: str) -> dict[str, str]:
    env_vars: dict[str, str] = {}

    if not os.path.exists(env_path):
        return env_vars

    with open(env_path, "r", encoding="utf-8") as env_file:
        for line in env_file:
            stripped = line.strip()
            if not stripped or stripped.startswith("#"):
                continue
            if "=" not in stripped:
                continue
            key, value = stripped.split("=", 1)
            env_vars[key] = value

    return env_vars


def _merge_local_env_template(
    source_env_path: str,
    destination_env_path: str,
    existing_env_vars: dict[str, str],
) -> None:
    with open(source_env_path, "r", encoding="utf-8") as source_file:
        source_lines = source_file.read().splitlines()

    template_keys: set[str] = set()
    merged_lines: list[str] = []

    for line in source_lines:
        stripped = line.strip()
        if (
            stripped
            and not stripped.startswith("#")
            and "=" in stripped
            and not stripped.startswith("{%")
            and not stripped.startswith("{{")
        ):
            key, value = stripped.split("=", 1)
            template_keys.add(key)
            if key in existing_env_vars:
                merged_lines.append(f"{key}={existing_env_vars[key]}")
                continue
            merged_lines.append(f"{key}={value}")
            continue

        merged_lines.append(line)

    additional_keys = [key for key in existing_env_vars if key not in template_keys]
    if additional_keys:
        if merged_lines and merged_lines[-1].strip() != "":
            merged_lines.append("")
        merged_lines.append("# Preserved custom values from previous .env")
        for key in additional_keys:
            merged_lines.append(f"{key}={existing_env_vars[key]}")

    merged_content = "\n".join(merged_lines)
    if merged_content and not merged_content.endswith("\n"):
        merged_content += "\n"

    with open(destination_env_path, "w", encoding="utf-8") as destination_file:
        destination_file.write(merged_content)


def _backup_local_deploy_files(
    project_path: str,
    deploy_path: str,
    docker_compose_target_path: str,
    local_env_target_path: str,
) -> str | None:
    paths_to_backup = [
        deploy_path,
        docker_compose_target_path,
        local_env_target_path,
    ]
    existing_paths = [path for path in paths_to_backup if os.path.exists(path)]
    if not existing_paths:
        return None

    backup_root = os.path.join(project_path, BACKUP_DIRECTORY)
    backup_timestamp = datetime.now().strftime("%Y%m%d-%H%M%S")
    backup_path = os.path.join(backup_root, backup_timestamp)
    os.makedirs(backup_path, exist_ok=False)

    for source_path in existing_paths:
        destination_path = os.path.join(backup_path, os.path.basename(source_path))
        if os.path.isdir(source_path):
            shutil.copytree(source_path, destination_path)
        else:
            shutil.copy2(source_path, destination_path)

    return backup_path


def setup_local_deploy(
    project_path: str,
    include_headscale: bool = False,
    base_domain: str | None = None,
    regenerate: bool = False,
    backup: bool = True,
) -> None:
    project_path = os.path.abspath(os.path.expanduser(project_path))

    deploy_path = os.path.join(project_path, ".deploy")
    docker_compose_template_path = os.path.join(deploy_path, "docker-compose.yml")
    docker_compose_target_path = os.path.join(project_path, "docker-compose.yml")
    local_env_template_path = os.path.join(deploy_path, ".env")
    local_env_target_path = os.path.join(project_path, ".env")

    selected_base_domain = base_domain
    if selected_base_domain is None:
        selected_base_domain = Prompt.ask(
            f"Base domain (example: {DEFAULT_ENV_VALUES['BASE_DOMAIN']})",
            default=DEFAULT_ENV_VALUES["BASE_DOMAIN"],
        )

    generated_hosts = _build_hosts_from_domain(selected_base_domain)
    public_web_host = generated_hosts["PUBLIC_WEB_HOST"]
    public_api_host = generated_hosts["PUBLIC_API_HOST"]
    headscale_server_url = generated_hosts["HEADSCALE_SERVER_URL"]
    headscale_internal_domain = generated_hosts["HEADSCALE_INTERNAL_DOMAIN"]
    nexus_api_url = _build_nexus_api_url(public_api_host=public_api_host)

    _print_domain_recap(
        base_domain=selected_base_domain,
        public_api_host=public_api_host,
        public_web_host=public_web_host,
        nexus_api_url=nexus_api_url,
        include_headscale=include_headscale,
        headscale_server_url=headscale_server_url,
        headscale_internal_domain=headscale_internal_domain,
    )

    template_values = {
        **DEFAULT_ENV_VALUES,
        **generated_hosts,
        "NEXUS_API_URL": nexus_api_url,
        "INCLUDE_HEADSCALE": include_headscale,
        "POSTGRES_PASSWORD": str(uuid4()),
        "MINIO_ROOT_PASSWORD": str(uuid4()),
        "RABBITMQ_PASSWORD": str(uuid4()),
        "FUSEKI_ADMIN_PASSWORD": str(uuid4()),
    }

    existing_env_vars = _read_env_vars(local_env_target_path)

    if regenerate:
        backup_path = None
        if backup:
            backup_path = _backup_local_deploy_files(
                project_path=project_path,
                deploy_path=deploy_path,
                docker_compose_target_path=docker_compose_target_path,
                local_env_target_path=local_env_target_path,
            )
            if backup_path is not None:
                print(f"Backed up previous local deploy files to {backup_path}")

        if os.path.exists(deploy_path):
            shutil.rmtree(deploy_path)

    if regenerate or not os.path.exists(deploy_path):
        os.makedirs(deploy_path, exist_ok=False)

        copier = Copier(
            templates_path=os.path.join(
                os.path.dirname(naas_abi_cli.__file__), "cli/deploy/templates/local"
            ),
            destination_path=deploy_path,
        )
        copier.copy(values=template_values)

    if os.path.exists(docker_compose_template_path):
        if regenerate and os.path.exists(docker_compose_target_path):
            os.remove(docker_compose_target_path)

        if os.path.exists(docker_compose_target_path):
            os.remove(docker_compose_template_path)
        else:
            shutil.move(docker_compose_template_path, docker_compose_target_path)

    if os.path.exists(local_env_template_path):
        if regenerate:
            _merge_local_env_template(
                source_env_path=local_env_template_path,
                destination_env_path=local_env_target_path,
                existing_env_vars=existing_env_vars,
            )
        else:
            _append_local_env_once(local_env_template_path, local_env_target_path)
        os.remove(local_env_template_path)

    for key, value in DEFAULT_ENV_VALUES.items():
        if not include_headscale and key.startswith("HEADSCALE_"):
            continue
        if key in {"PUBLIC_WEB_HOST", "PUBLIC_API_HOST"}:
            continue
        if key in {"BASE_DOMAIN", "HEADSCALE_SERVER_URL", "HEADSCALE_INTERNAL_DOMAIN"}:
            continue
        _ensure_env_var(local_env_target_path, key, value)

    _ensure_env_var(local_env_target_path, "PUBLIC_WEB_HOST", public_web_host)
    _ensure_env_var(local_env_target_path, "PUBLIC_API_HOST", public_api_host)
    if include_headscale:
        _ensure_env_var(
            local_env_target_path, "HEADSCALE_SERVER_URL", headscale_server_url
        )
        _ensure_env_var(
            local_env_target_path,
            "HEADSCALE_INTERNAL_DOMAIN",
            headscale_internal_domain,
        )

    for key in RANDOM_ENV_KEYS:
        _ensure_env_var(local_env_target_path, key, str(uuid4()))

    persisted_public_api_host = (
        _get_env_var(local_env_target_path, "PUBLIC_API_HOST")
        or DEFAULT_ENV_VALUES["PUBLIC_API_HOST"]
    )
    public_api_scheme = _get_env_var(local_env_target_path, "PUBLIC_API_SCHEME")
    _ensure_env_var(
        local_env_target_path,
        "NEXUS_API_URL",
        _build_nexus_api_url(
            public_api_host=persisted_public_api_host,
            public_api_scheme=public_api_scheme,
        ),
    )

    if include_headscale:
        _copy_headscale_templates(deploy_path, values=template_values)
        _ensure_headscale_service(docker_compose_target_path)



import os

import click


@click.command("init")
@click.argument("path")
def init(path: str):
    if path == ".":
        path = os.getcwd()

    os.makedirs(path, exist_ok=True)
    # os.exec(f"cd {path} && uv init .")



import os

import click
import naas_abi_cli
from naas_abi_cli.cli.utils.Copier import Copier

from .agent import new_agent
from .integration import new_integration
from .new import new
from .orchestration import new_orchestration
from .pipeline import new_pipeline
from .utils import to_kebab_case, to_pascal_case, to_snake_case
from .workflow import new_workflow


@new.command("module")
@click.argument("module_name", required=True)
@click.argument("module_path", required=False, default=".")
def _new_module(module_name: str, module_path: str = "."):
    new_module(module_name, module_path)


def new_module(module_name: str, module_path: str = ".", quiet: bool = False):
    module_name = to_kebab_case(module_name)

    if module_path == ".":
        module_path = os.path.join(os.getcwd(), to_snake_case(module_name))
    else:
        module_path = os.path.join(module_path, to_snake_case(module_name))

    if not os.path.exists(module_path):
        os.makedirs(module_path, exist_ok=True)
    elif len(os.listdir(module_path)) > 0:
        print(f"Folder {module_path} already exists and is not empty.")
        exit(1)

    copier = Copier(
        templates_path=os.path.join(
            os.path.dirname(naas_abi_cli.__file__), "cli/new/templates/module"
        ),
        destination_path=module_path,
    )

    copier.copy(
        values={
            "module_name": module_name,
            "module_name_snake": to_snake_case(module_name),
            "module_name_pascal": to_pascal_case(module_name),
        }
    )

    new_agent(
        module_name,
        os.path.join(module_path, "agents"),
        extra_values={"module_name_snake": to_snake_case(module_name)},
    )

    new_integration(
        module_name,
        os.path.join(module_path, "integrations"),
        extra_values={"module_name_snake": to_snake_case(module_name)},
    )
    
    new_pipeline(
        module_name,
        os.path.join(module_path, "pipelines"),
        extra_values={"module_name_snake": to_snake_case(module_name)},
    )
    
    new_workflow(
        module_name,
        os.path.join(module_path, "workflows"),
        extra_values={"module_name_snake": to_snake_case(module_name)},
    )
    
    new_orchestration(
        module_name,
        os.path.join(module_path, "orchestrations"),
        extra_values={"module_name_snake": to_snake_case(module_name)},
    )

    if not quiet:
        print(f"\nModule '{module_name}' has been created at:\n  {module_path}\n")
        print("To enable this module, add the following to your config.yaml:\n")
        print("modules:")
        print(f"  - path: {module_path}")
        print("    enabled: true\n")



import importlib

import click
import pytest


stack_module = importlib.import_module("naas_abi_cli.cli.stack")


def test_start_stack_opens_service_portal_when_compose_succeeds(monkeypatch) -> None:
    opened: list[str] = []

    monkeypatch.setattr(stack_module, "run_compose", lambda args: None)
    monkeypatch.setattr(stack_module, "_get_error_services", lambda: [])
    monkeypatch.setattr(stack_module.webbrowser, "open", lambda url: opened.append(url))

    stack_module._start_stack()

    assert opened == [stack_module.SERVICE_PORTAL_URL]


def test_start_stack_retries_compose_up_before_succeeding(monkeypatch) -> None:
    attempts = {"count": 0}
    messages: list[str] = []

    def _flaky_compose(args: list[str]) -> None:
        assert args == ["up", "-d"]
        attempts["count"] += 1
        if attempts["count"] < 3:
            raise click.ClickException("temporary failure")

    monkeypatch.setattr(stack_module, "run_compose", _flaky_compose)
    monkeypatch.setattr(stack_module, "_get_error_services", lambda: [])
    monkeypatch.setattr(stack_module.webbrowser, "open", lambda url: True)
    monkeypatch.setattr(
        stack_module.click, "echo", lambda message: messages.append(message)
    )

    stack_module._start_stack()

    assert attempts["count"] == 3
    assert any("Retrying" in message for message in messages)


def test_start_stack_fails_after_two_retries(monkeypatch) -> None:
    attempts = {"count": 0}

    def _always_failing_compose(args: list[str]) -> None:
        assert args == ["up", "-d"]
        attempts["count"] += 1
        raise click.ClickException("still failing")

    monkeypatch.setattr(stack_module, "run_compose", _always_failing_compose)
    monkeypatch.setattr(stack_module, "_get_error_services", lambda: [])

    with pytest.raises(click.ClickException, match="still failing"):
        stack_module._start_stack()

    assert attempts["count"] == 3


def test_start_stack_retries_when_container_is_in_error(monkeypatch) -> None:
    attempts = {"count": 0}
    checks = {"count": 0}
    messages: list[str] = []

    def _compose_ok(args: list[str]) -> None:
        assert args == ["up", "-d"]
        attempts["count"] += 1

    def _flaky_errors() -> list[str]:
        checks["count"] += 1
        if checks["count"] < 3:
            return ["rabbitmq"]
        return []

    monkeypatch.setattr(stack_module, "run_compose", _compose_ok)
    monkeypatch.setattr(stack_module, "_get_error_services", _flaky_errors)
    monkeypatch.setattr(stack_module.webbrowser, "open", lambda url: True)
    monkeypatch.setattr(
        stack_module.click, "echo", lambda message: messages.append(message)
    )

    stack_module._start_stack()

    assert attempts["count"] == 3
    assert checks["count"] == 3
    assert any("containers are in error" in message for message in messages)


def test_start_stack_fails_after_container_error_retries_exhausted(monkeypatch) -> None:
    attempts = {"count": 0}
    logged: list[list[str]] = []

    def _compose_ok(args: list[str]) -> None:
        assert args == ["up", "-d"]
        attempts["count"] += 1

    monkeypatch.setattr(stack_module, "run_compose", _compose_ok)
    monkeypatch.setattr(stack_module, "_get_error_services", lambda: ["rabbitmq"])
    monkeypatch.setattr(
        stack_module, "_show_error_logs", lambda services: logged.append(services)
    )

    with pytest.raises(
        click.ClickException,
        match="containers are in error after startup: rabbitmq",
    ):
        stack_module._start_stack()

    assert attempts["count"] == 3
    assert logged == [["rabbitmq"]]



from fastapi import FastAPI
from naas_abi_core.module.Module import (
    BaseModule,
    ModuleConfiguration,
    ModuleDependencies,
)

# from naas_abi_core.services.object_storage.ObjectStorageService import ObjectStorageService
# from naas_abi_core.services.secret.Secret import Secret
# from naas_abi_core.services.triple_store.TripleStoreService import TripleStoreService
# from naas_abi_core.services.vector_store.VectorStoreService import VectorStoreService
# from naas_abi_core.services.bus.BusService import BusService
# from naas_abi_core.services.keyvalue.KeyValueService import KeyValueService


class ABIModule(BaseModule):
    dependencies: ModuleDependencies = ModuleDependencies(
        modules=[
            "naas_abi_marketplace.ai.chatgpt",
        ],
        services=[
            # Secret,
            # TripleStoreService,
            # ObjectStorageService,
            # VectorStoreService,
            # BusService,
            # KeyValueService,
        ],
    )

    class Configuration(ModuleConfiguration):
        pass
        # example: str

    # on_initialized is called by the engine after all modules and services have been fully loaded.
    # At this point, you can safely access other modules and services through the engine's interfaces.
    # Override this method to implement any post-initialization logic your module requires.
    def on_initialized(self):
        super().on_initialized()

    # The on_load method is invoked during initial module loading by the engine.
    # At this point, avoid accessing services or other modules, as they have not been loaded yet.
    # Place any logic here that must occur right as the module is loaded, before initialization.
    # You can see it as the constructor of the module.
    def on_load(self):
        super().on_load()

    # Optional FastAPI integration hook.
    # This mirrors how `naas_abi` wires API settings and services into app.state.
    # Override and adapt to your module if you expose HTTP routes.
    def api(self, app: FastAPI) -> None:
        # Example: expose services to your API layer.
        # app.state.object_storage = self.engine.services.object_storage
        # app.state.secret_service = self.engine.services.secret
        # app.state.triple_store = self.engine.services.triple_store
        # app.state.vector_store = self.engine.services.vector_store
        # app.state.bus_service = self.engine.services.bus
        # app.state.key_value_service = self.engine.services.kv

        # Example: mount your FastAPI routes/app factory.
        # from your_module.apps.api.app.main import create_app
        # create_app(app)
        pass
